// ##########################################################################
// ####                                                                  ####
// ####                    Master Thesis Implementation                  ####
// ####  Management of Layered Variable Bitrate Multimedia Streams over  ####
// ####                  DiffServ with A Priori Knowledge                ####
// ####                                                                  ####
// #### ================================================================ ####
// ####                                                                  ####
// ####                                                                  ####
// #### Abstract Layer Description Implementation                        ####
// ####                                                                  ####
// #### Version 1.00  --  October 10, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 2000 Thomas Dreibholz                              ####
// #### University of Bonn, Department of Computer Science IV            ####
// #### EMail: Dreibholz@bigfoot.com                                     ####
// #### WWW:   http://www.bigfoot.com/~dreibholz/diplom/index.html       ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "abstractlayerdescription.h"


namespace Coral {


// ###### Constructor #######################################################
AbstractLayerDescription::AbstractLayerDescription()
{
   PktHeaderSize    = 28;
   PktMaxSize       = 1500;
   Bandwidth        = 0;
   Flags            = LF_BaseLayer;
   BandwidthLimit   = (cardinal)-1;
   MaxTransferDelay = HUGE_VAL;
   MaxLossRate      = HUGE_VAL;
   MaxJitter        = HUGE_VAL;
   BufferDelay      = 1;
}


// ###### Desstructor #######################################################
AbstractLayerDescription::~AbstractLayerDescription()
{
}


// ###### Get packet rate ###################################################
cardinal AbstractLayerDescription::getPacketRate(const double frameRate) const
{
   return((cardinal)ceil(frameRate) *
             getPacketCountForSize(
                frameRate,
                (cardinal)floor((double)Bandwidth / frameRate)));
}


// ###### Translate raw into payload frame size #############################
cardinal AbstractLayerDescription::rawToPayload(const double   frameRate,
                                                const cardinal raw) const
{
   const cardinal maxFrameCount   = getMaxFrameCountForDelay(frameRate,BufferDelay);
   const cardinal maxFramePackets = (maxFrameCount > 1) ? maxFrameCount - 1 : 0;
   const cardinal frameSize       = raw - (maxFramePackets * PktHeaderSize);
   const cardinal payloadPackets  = (cardinal)ceil((double)frameSize /
                                                      (double)PktMaxSize);

   const cardinal payload = frameSize - (payloadPackets * PktHeaderSize);

/*
   if(raw != payloadToRaw(payload)) {
      cerr << "ERROR: Raw to Payload!" << endl;
      exit(1);
   }
*/

   return(payload);
}


// ###### Calculate packet rate for frame size ##############################
cardinal AbstractLayerDescription::frameSizeToPacketRate(
                                      const double   frameRate,
                                      const cardinal frameSize) const
{
   return((cardinal)ceil((double)getPacketCountForSize(
                                    frameRate, frameSize) * frameRate));
}


// ###### Set buffer delay ##################################################
cardinal AbstractLayerDescription::setBufferDelay(const cardinal bufferDelay)
{
   if(BufferDelay < 1) {
      BufferDelay = 1;
   }
   else {
      BufferDelay = bufferDelay;
   }
   return(BufferDelay);
}


}
