#include "../diplom/system.h"

#include <fstream.h>



const cardinal frequencies[2][3]=
{
  {44100,48000,32000}, // MPEG 1
  {22050,24000,16000}  // MPEG 2
};

const cardinal bitrate[2][3][15]=
{
  // MPEG 1
  {{0,32,64,96,128,160,192,224,256,288,320,352,384,416,448},
   {0,32,48,56,64,80,96,112,128,160,192,224,256,320,384},
   {0,32,40,48,56,64,80,96,112,128,160,192,224,256,320}},

  // MPEG 2
  {{0,32,48,56,64,80,96,112,128,144,160,176,192,224,256},
   {0,8,16,24,32,40,48,56,64,80,96,112,128,144,160},
   {0,8,16,24,32,40,48,56,64,80,96,112,128,144,160}}
};


cardinal layer3slots;
cardinal layer;
cardinal protection;
cardinal bitrateindex;
cardinal padding;
cardinal extendedmode;
cardinal framesize;
cardinal frames = 0;

enum _mpegversion {mpeg1,mpeg2}                                  version;
enum _mode        {fullstereo,jocardinal,dual,single}            mode;
enum _frequency   {frequency44100,frequency48000,frequency32000} frequency;



// ###### Read a byte #######################################################
cardinal readByte(ifstream* loader)
{
   unsigned char c;
   loader->read(&c,1);
   return((cardinal)c);
}


// ###### Skip given number of bytes ########################################
bool skipBytes(ifstream* loader, const cardinal bytes)
{
   if(bytes > 0) {
      char buffer[bytes];
      loader->read(&buffer,bytes);
   }
   return(loader->good());
}


// ###### Check, if header is valid #########################################
bool isValidHeader(const cardinal mpeg,
                   const cardinal mylayer,
                   const cardinal brindex,
                   const cardinal sfreq)
{
   // ====== Check if mpeg, layer and bitrateindex are valid ================
   if( (mylayer < 1 || mylayer > 3)  || (mpeg  < 1 || mpeg  > 2) ||
       (brindex < 1 || brindex > 14) || (sfreq < 0 || sfreq > 2)) {
      return(false);
   }

   // ====== Print information ==============================================
   static bool firstCall = false;
   if(!firstCall) {
      char buffer[200];
      snprintf(buffer, sizeof(buffer),
              "MPEG-%d Layer %d, %d BPS, %d Hz\n",
              mpeg, mylayer,
              1000 * bitrate[mpeg - 1][mylayer - 1][brindex],
              frequencies[mpeg - 1][sfreq]);
      cerr << buffer;
      firstCall = true;
   }

   return(true);
}


// ###### Get next frame ####################################################
bool getNextFrame(ifstream* loader)
{
   // ====== Synchronize ====================================================
   cardinal c;
   bool okay;
   okay=false;
   do {
      if(loader->eof()) {
         return(false);
      }
      if((c = readByte(loader)) < 0) {
         break;
      }
      if(c == 0xff) {
         while(!okay) {
            if((c = readByte(loader)) < 0) {
               okay=true;
               break;
            }
            if((c & 0xf0) == 0xf0) {
               okay=true;
               break;
            }
            else if(c != 0xff) {
               break;
            }
         }
      }
   } while(!okay);
   if(c < 0) {
      return(true);
   }


   // ====== Get header data ================================================
   c &= 0xf;                        /* c = 0 0 0 0 bit13 bit14 bit15 bit16 */
   protection = c & 1;
   layer      = 4 - ((c>>1) & 3);   /* 1..4, 4=invalid */
   version    = (_mpegversion)((c >> 3) ^ 1);
   c = ((readByte(loader))) >> 1;   /* c = 0 bit17 .. bit23 */
   padding      = (c&1);             c>>=1;
   frequency    = (_frequency)(c&3); c>>=2;
   bitrateindex = (cardinal)c;

   c = ((cardinal)(readByte(loader))) >> 4; /* c = 0 0 0 0 bit25 bit26 bit27 bit27 bit28 */
   extendedmode = c & 3;
   mode         = (_mode)(c >> 2);

   if(!(isValidHeader(version + 1,layer,bitrateindex,frequency))) {
      cerr << "WARNING: getNextFrame() - Bad header format!" << endl;
      return(true);
   }

   if(layer == 1) {
     framesize = (12000 * bitrate[version][0][bitrateindex]) /
                    frequencies[version][frequency];
     if(frequency == frequency44100 && padding) {
        framesize++;
     }
     framesize <<= 2;
   }
   else {
      framesize = (144000 * bitrate[version][layer-1][bitrateindex]) /
                     (frequencies[version][frequency] << version);
      if(padding) {
         framesize++;
      }
      if(layer == 3) {
         if(version) {
            layer3slots = framesize - ((mode == single) ? 9 : 17)
                                    - (protection ? 0 : 2)
                                    - 4;
         }
         else {
            layer3slots = framesize - ((mode == single) ? 17 : 32)
                                    - (protection ? 0 : 2)
                                    - 4;
         }
      }
   }
   frames++;


   // ====== Print frame size ===============================================
   cout << framesize << endl;


   // ====== Skip frame data ================================================
   if(!skipBytes(loader,framesize - 4)) {
      return(false);
   }
   if(!protection) {
      readByte(loader);
      readByte(loader);
   }
   return(true);
}



// ###### Main program ######################################################
int main(int argc, char**argv)
{
   if(argc < 2) {
      cerr << "Usage: " << argv[0] << " [MP3 file]" << endl;
      exit(1);
   }


   ifstream is(argv[1]);
   if(is.good()) {
      while(getNextFrame(&is));
      cerr << frames << " frames read." << endl;
   }
   else {
      cerr << "ERROR: Unable to load " << argv[1] << "!" << endl;
      exit(1);
   }
   return(0);
}
