// ##########################################################################
// ####                                                                  ####
// ####                    Master Thesis Implementation                  ####
// ####  Management of Layered Variable Bitrate Multimedia Streams over  ####
// ####                  DiffServ with A Priori Knowledge                ####
// ####                                                                  ####
// #### ================================================================ ####
// ####                                                                  ####
// ####                                                                  ####
// #### MPEG Writer QoS Description Implementation                       ####
// ####                                                                  ####
// #### Version 1.00  --  October 10, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 2000 Thomas Dreibholz                              ####
// #### University of Bonn, Department of Computer Science IV            ####
// #### EMail: Dreibholz@bigfoot.com                                     ####
// #### WWW:   http://www.bigfoot.com/~dreibholz/diplom/index.html       ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "mpegwriterqosdescription.h"
#include "utilityfunction.h"


namespace Coral {


// ###### Constructor #######################################################
MPEGWriterQoSDescription::MPEGWriterQoSDescription(
                             const TraceConfiguration* traceConfiguration)
{
   Config = traceConfiguration;
}


// ###### Calculate utilization for given layer bandwidths ##################
double MPEGWriterQoSDescription::calculateUtilizationForLayerBandwidths(
                                    const double    frameRate,
                                    const cardinal  layers,
                                    const cardinal* bandwidth) const
{
   // ====== Initialize =====================================================
   if(layers < getLayers()) {
      return(-1.0);
   }
   double utilization = 0.0;
   double weight      = 0.0;
   bool   ok          = true;


   // ====== Calculate utilization for each layer ===========================
   for(cardinal i = 0;i < getLayers();i++) {
      const AbstractLayerDescription* ald = getLayer(i);
      const cardinal frameSize = ald->bandwidthToFrameSize(frameRate,bandwidth[i]);
      const double w = Config->Layer[i].FrameSizeUtilizationWeight;
      weight += w;
      if(frameSize < ald->getMinFrameSize(frameRate)) {
         if(!(ald->getFlags() & AbstractLayerDescription::LF_ExtensionLayer)) {
            ok = false;
            break;
         }
         else if(frameSize > 0) {
            cerr << "WARNING: AbstractQoSDescription::calculateUtilizationForLayerBandwidths() -"
                    "Senseless allocation to extension layer?!" << endl;
         }
      }
      else {
         if(layers > Config->MaxLayers) {
            cerr << "ERROR: MPEGWriterQoSDescription::calculateUtilizationForLayerBandwidths() - "
                 << "Too few layers defined in configuration!" << endl;
            exit(1);
         }
         utilization += w *
            evaluateUtilityFunction(
               Config->Layer[i].FrameSizeUtilizationType,
               ald->getFrameSizeScaleFactorForSize(frameRate,frameSize),
               (double*)&Config->Layer[i].FrameSizeUtilizationConstant,
               Config->Layer[i].FrameSizeUtilizationConstants);
      }
   }


   // ====== Calculate global utilization ===================================
   if(ok == true) {
      weight      += Config->FrameRateUtilizationWeight;
      utilization += Config->FrameRateUtilizationWeight *
                        evaluateUtilityFunction(
                           Config->FrameRateUtilizationType,
                           getFrameRateScaleFactorForRate(frameRate),
                           (double*)&Config->FrameRateUtilizationConstant,
                           Config->FrameRateUtilizationConstants);
      utilization /= weight;
   }
   else {
      utilization = -1.0;
   }
   return(utilization);
}


}
