// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Pinger Host                                                      ####
// ####                                                                  ####
// #### Version 1.00  --  September 17, 2000                             ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####                     EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#ifndef PINGERHOST_H
#define PINGERHOST_H


#include "system.h"
#include "socket.h"
#include "strings.h"
#include "internetaddress.h"
#include "timedthread.h"


namespace Coral {


/**
  * This structure contains internal information for RoundTripTimePinger.
  *
  * @short   PingerHost
  * @author  Thomas Dreibholz (Dreibholz@bigfoot.com)
  * @version 1.0
  */
struct PingerHost
{
   /**
     * InternetAddress to send ping to.
     */
   InternetAddress Address;
   
   /**
     * The ping address in text format.
     */
   String AddressString;

   /**
     * Timestamp of last sent ping.
     */
   card64 LastPingTimeStamp;

   /**
     * Timestamp of last received ping.
     */
   card64 LastEchoTimeStamp;

   /**
     * Round trip time.
     */
   cardinal RoundTripTime;

   /**
     * Maximum raw round trip time (directly calculated from packet).
     */
   cardinal MaxRawRoundTripTime;

   /**
     * Sequence number.
     */
   card16 SeqNum;

   /**
     * Traffic class.
     */
   card8 TrafficClass;

   /**
     * Does this address use IPv6?
     */
   bool IsIPv6;
};


/**
  * Operator "==".
  */
inline int operator==(const PingerHost& ph1, const PingerHost& ph2);

/**
  * Operator "!=".
  */
inline int operator==(const PingerHost& ph1, const PingerHost& ph2);

/**
  * Operator "<".
  */
inline int operator<(const PingerHost& ph1, const PingerHost& ph2);

/**
  * Operator ">".
  */
inline int operator>(const PingerHost& ph1, const PingerHost& ph2);


}


#include "pingerhost.icc"


#endif
