#include "system.h"
#include "tdtf.h"
#include "tracearray.h"
#include "mpegtracearray.h"
#include "tdtfwriter.h"
#include "tdtfreader.h"


#include <sys/time.h>


TraceConfiguration gConfig = {
   25,

   0,   // ExtLayers
   0.0, // FakeE1
   0.0, // FakeE2

   30,
   3000000.0,

   1.0,2.0,3.0,

   {"Test"},
   {"Copyright 2000, Thomas Dreibholz"},

   {
      {
         3.0,
         1.9, 1.8, 1.3,
         12,
         4
      },
      {
         2.0,
         1.7, 1.6, 1.2,
         12,
         4
      },
      {
         1.0,
         1.5, 1.4, 1.1,
         12,
         4
      },

      {
         1.0,
         1.9, 1.8, 1.3,
         12,
         4
      },
      {
         1.0,
         1.7, 1.6, 1.2,
         12,
         4
      },
      {
         1.0,
         1.5, 1.4, 1.1,
         12,
         4
      },

      {
         1.0,
         1.9, 1.8, 1.3,
         12,
         4
      },
      {
         1.0,
         1.7, 1.6, 1.2,
         12,
         4
      },
      {
         1.0,
         1.5, 1.4, 1.1,
         12,
         4
      }
   }
};


// ###### Main program ######################################################
int main(int argc, char* argv[])
{
   loadTraceConfiguration("test.cfg",gConfig);

   cout << "TDTF Trace Generator - Copyright (C) 2000 Thomas Dreibholz" << endl
        << "----------------------------------------------------------" << endl
        << endl;

   cout << gConfig << endl;

   int g= 1 ;

   if(g) {
      MPEGTraceArray array(gConfig);
      if(array.load("test2.t",
                    "IBBPBBPBBPBB",
                    gConfig.FrameRate,
                    gConfig.ExtLayers,
                    gConfig.FakeE1,
                    gConfig.FakeE2) == true) {

         array.Frames = 50;

         cout << array << endl;

         TDTFWriter w;
         w.generate("test.out",&array,gConfig);
      }
      else {
         exit(1);
      }
   }


   TDTFReader tr;
   if(tr.open("test.out")) {
      const TDTFPrefix* tp = tr.getTDTFPrefix();
      const TDTFSuffix* ts = tr.getTDTFSuffix();
      char str[256];


      cout << "TDTF Trace Printer - Copyright (C) 2000 Thomas Dreibholz" << endl
           << "--------------------------------------------------------" << endl
           << endl;
      cout << "Title            = " << tp->Title     << endl;
      cout << "Copyright        = " << tp->Copyright << endl;

      time_t timeStamp1  = translate64(tp->TimeStamp) / 1000000;
      struct tm* timeptr = localtime(&timeStamp1);
      strftime((char*)&str,256,"%A, %B %d %G, %T",timeptr);
      cout << "Generation Start = " << str << endl;

      time_t timeStamp2  = translate64(ts->TimeStamp) / 1000000;
      timeptr   = localtime(&timeStamp2);
      strftime((char*)&str,256,"%A, %B %d %G, %T",timeptr);
      cout << "Generation End   = " << str << endl;

      time_t diff = timeStamp2 - timeStamp1;
      snprintf((char*)&str,sizeof(str),"%ld:%02ld:%02ld",
                  diff / 3600, (diff / 60) % 60, diff % 60);
      cout << "Duration         = " << str << endl;

      cout << "Frame Rates      = {";
      double frameRate = tr.getMinFrameRate();
      while(frameRate != tr.getMaxFrameRate()) {
         cout << frameRate << " ";
         frameRate = tr.getNextFrameRateForRate(frameRate);
      }
      cout << tr.getMaxFrameRate() << "}" << endl;

      cout << endl;


      tr.print(cout,25.0,true,false);
   }
   return(0);
}
