// ##########################################################################
// ####                                                                  ####
// ####                    Master Thesis Implementation                  ####
// ####  Management of Layered Variable Bitrate Multimedia Streams over  ####
// ####                  DiffServ with A Priori Knowledge                ####
// ####                                                                  ####
// #### ================================================================ ####
// ####                                                                  ####
// ####                                                                  ####
// #### Thomas Dreibholz's Trace File (TDTF) Implementation              ####
// ####                                                                  ####
// #### Version 1.00  --  October 10, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 2000 Thomas Dreibholz                              ####
// #### University of Bonn, Department of Computer Science IV            ####
// #### EMail: Dreibholz@bigfoot.com                                     ####
// #### WWW:   http://www.bigfoot.com/~dreibholz/diplom/index.html       ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "tdtf.h"


namespace Coral {


// ###### Get constraint for given delay ####################################
cardinal EmpiricalEnvelope::getConstraint(const cardinal bufferDelay) const
{
   const cardinal pairs = translate16(Pairs);


   // ====== No approximation necessary if interval is in table =============
   for(cardinal i = 0;i < pairs;i++) {
      if(translate16(Pair[i].Interval) == bufferDelay) {
         return(translate32(Pair[i].Sum));
      }
   }


   // ====== Calculate indices for D-BIND approx. calculation ([KZ97]) ======
   integer k0 = 0;
   for(integer i = 0;i < (integer)pairs;i++) {
      const cardinal interval = translate16(Pair[i].Interval);
      if(interval >= bufferDelay) {
         k0 = max(i - 1,0);
         break;
      }
   }
   if(k0 >= (integer)pairs) {
      k0 = pairs - 1;
   }
   integer k1 = (integer)pairs - 1;
   for(integer i = k1;i >= 0;i--) {
      const cardinal interval = translate16(Pair[i].Interval);
      if(interval <= bufferDelay) {
         k1 = min(i + 1,(integer)pairs - 1);
         break;
      }
   }

   // ====== Do D-BIND approximation ([KZ97]) ===============================
   const double i0 = (double)translate16(Pair[k0].Interval);
   const double i1 = (double)translate16(Pair[k1].Interval);
   const double r0 = (double)translate32(Pair[k0].Sum);
   const double r1 = (double)translate32(Pair[k1].Sum);
   const double n = ((r1 - r0) * (double)(bufferDelay - i0)) / (i1 - i0) + r0;
   return((cardinal)ceil(n));
}


}
