// ##########################################################################
// ####                                                                  ####
// ####                    Master Thesis Implementation                  ####
// ####  Management of Layered Variable Bitrate Multimedia Streams over  ####
// ####                  DiffServ with A Priori Knowledge                ####
// ####                                                                  ####
// #### ================================================================ ####
// ####                                                                  ####
// ####                                                                  ####
// #### TDTF Writer                                                      ####
// ####                                                                  ####
// #### Version 1.00  --  October 10, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 2000 Thomas Dreibholz                              ####
// #### University of Bonn, Department of Computer Science IV            ####
// #### EMail: Dreibholz@bigfoot.com                                     ####
// #### WWW:   http://www.bigfoot.com/~dreibholz/diplom/index.html       ####
// ####                                                                  ####
// ##########################################################################


#ifndef TDTFWRITER_H
#define TDTFWRITER_H


#include "system.h"
#include "tracearray.h"
#include "traceconfiguration.h"


namespace Coral {


/**
  * This class is a writer for TDTF files.
  *
  * @short   TDTF Writer
  * @author  Thomas Dreibholz (Dreibholz@bigfoot.com)
  * @version 1.0
  */
class TDTFWriter
{
   // ====== Constructor/Destructor =========================================
   public:
   /**
     * Constructor.
     */
   TDTFWriter();

   /**
     * Destructor.
     */
   ~TDTFWriter();


   // ====== Generate TDTF file =============================================
   /**
     * Generator TDTF file.
     *
     * @param name File name for TDTF file to generate.
     * @param originalTraceArray TraceArray containing input trace.
     * @param config Configuration for the TDTF trace generator.
     */
   void generate(const char*               name,
                 TraceArray*               originalTraceArray,
                 const TraceConfiguration& config);


   // ====== Private data ===================================================
   private:
   cardinal writeTrace();
   cardinal writeIntervals();


   FILE*              OutputFile;
   TraceArray*        TArray;
   TraceConfiguration Config;
};


}


#endif
