// ##########################################################################
// ####                                                                  ####
// ####                    Master Thesis Implementation                  ####
// ####  Management of Layered Variable Bitrate Multimedia Streams over  ####
// ####                  DiffServ with A Priori Knowledge                ####
// ####                                                                  ####
// #### ================================================================ ####
// ####                                                                  ####
// ####                                                                  ####
// #### Trace Decoder Interface                                          ####
// ####                                                                  ####
// #### Version 1.00  --  October 10, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 2000 Thomas Dreibholz                              ####
// #### University of Bonn, Department of Computer Science IV            ####
// #### EMail: Dreibholz@bigfoot.com                                     ####
// #### WWW:   http://www.bigfoot.com/~dreibholz/diplom/index.html       ####
// ####                                                                  ####
// ##########################################################################


#ifndef TRACEDECODERINTERFACE_H
#define TRACEDECODERINTERFACE_H


#include "system.h"
#include "decoderinterface.h"
#include "range.h"


namespace Coral {


/**
  * This class is the interface for a trace decoder.
  *
  * @short   Trace Decoder Interface
  * @author  Thomas Dreibholz (Dreibholz@bigfoot.com)
  * @version 1.0
  */
class TraceDecoderInterface : virtual public DecoderInterface
{
   public:
   /**
     * Set frameRate.
     *
     * @param frameRate FrameRate.
     * @return FrameRate set.
     */
   virtual double getFrameRate() const = 0;

   /**
     * Get utilization.
     *
     * @return Utilization (out of [0,1]).
     */
   virtual double getUtilization() const = 0;

   /**
     * Get bandwidth range.
     *
     * @return Bandwidth range.
     */
   virtual Range<cardinal> getBandwidth() const = 0;

  /**
     * Get stream priority.
     *
     * @return Stream priority.
     */
   virtual int8 getStreamPriority() const = 0;

  /**
     * Get session priority.
     *
     * @return Session priority.
     */
   virtual int8 getSessionPriority() const = 0;
};


}


#endif
