// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Decoder Repository Interface                                     ####
// ####                                                                  ####
// #### Version 1.00  --  September 17, 2000                             ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####                     EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#ifndef DECODERREPOSITORYINTERFACE_H
#define DECODERREPOSITORYINTERFACE_H


#include "system.h"
#include "synchronizable.h"
#include "decoderinterface.h"


namespace Coral {


/**
  * This class is a repository for decoders.
  *
  * @short   Decoder Repository
  * @author  Thomas Dreibholz (Dreibholz@bigfoot.com)
  * @version 1.0
  */
class DecoderRepositoryInterface : virtual public DecoderInterface
{
   // ====== Select another decoder =========================================
   public: 
   /**
     * Select the decoder with the given TypeID to be the current decoder
     * of the repository.
     *
     * @param typeID Decoding's type ID.
     * @return true, if decoder for this TypeID was in the repository;
     * false otherwise.
     */
   virtual bool selectDecoderForTypeID(const card16 typeID) = 0;
   
   /**
     * Get DecoderInterface of the current decoder.
     *
     * @return Current decoder's DecoderInterface.     
     */
   virtual DecoderInterface* getCurrentDecoder() const = 0;
};


}


#endif
