// ##########################################################################
// ####                                                                  ####
// ####                    Master Thesis Implementation                  ####
// ####  Management of Layered Variable Bitrate Multimedia Streams over  ####
// ####                  DiffServ with A Priori Knowledge                ####
// ####                                                                  ####
// #### ================================================================ ####
// ####                                                                  ####
// ####                                                                  ####
// #### Trace Client                                                     ####
// ####                                                                  ####
// #### Version 1.00  --  October 10, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 2000 Thomas Dreibholz                              ####
// #### University of Bonn, Department of Computer Science IV            ####
// #### EMail: Dreibholz@bigfoot.com                                     ####
// #### WWW:   http://www.bigfoot.com/~dreibholz/diplom/index.html       ####
// ####                                                                  ####
// ##########################################################################


#include "socket.h"
#include "traceclient.h"
#include "breakdetector.h"
#include "strings.h"


#include <fstream.h>


using namespace Coral;


// Fast Break: Disable break detector to debug thread deadlocks
// #define FAST_BREAK


// Globals
TraceClient* client      = NULL;
ofstream*    gpScript    = NULL;
ofstream*    gpData      = NULL;


// ###### Clean up ##########################################################
void cleanUp(const cardinal exitCode = 0)
{
   if(client != NULL)
      delete client;
   if(exitCode == 0)
      cout << "Terminated!" << endl;
   exit(exitCode);
}


// ###### Initialize GNUplot output streams #################################
void initGNUplot(const char*    prefix,
                 const char*    info,
                 const cardinal layers,
                 const String&  address)
{
   const String scriptName = String(prefix) + ".gp";
   const String dataName   = String(prefix) + ".dat";
   time_t timeStamp = getMicroTime() / 1000000;
   const struct tm* timeptr = localtime(&timeStamp);
   char str[256];
   strftime((char*)&str,256,"%A, %B %d %G %T %Z",timeptr);
   String infoString = "Generated on " + (String)str;
   if(info) {
      infoString = infoString + " " + (String)info;
   }

   gpScript = new ofstream(scriptName.getData());
   if(gpScript != NULL) {
      if(gpScript->good()) {
         gpData = new ofstream(dataName.getData());
         if(gpData != NULL) {
            if(gpData->good()) {
               *gpScript << "set terminal postscript enhanced" << endl;
               *gpScript << "set encoding iso_8859_1" << endl;

               *gpScript << "set title \"RTP Trace Client " << address << " - Bytes Received\" \"Times-Roman,16\"" << endl;
               *gpScript << "set timestamp \"" << infoString << "\" 0,0 \"Times-Roman,12\"" << endl;
               *gpScript << "set ylabel \"Bytes [1]\"" << endl;
               *gpScript << "set xlabel \"Time [s]\"" << endl;
               *gpScript << "plot \"" << dataName << "\" using 1:"
                         << 1 + (layers + 1) << " title "
                         << "\"Total\" with lines";
               for(cardinal i = 0;i < layers;i++) {
                  *gpScript << ", \"" << dataName << "\" using 1:"
                            << 1 + (i + 1) << " title "
                            << "\"Layer #" << i << "\" with lines";
               }
               *gpScript << endl;

               *gpScript << "set title \"RTP Trace Client " << address << " - Packets Received\" \"Times-Roman,16\"" << endl;
               *gpScript << "set timestamp \"" << infoString << "\" 0,0 \"Times-Roman,12\"" << endl;
               *gpScript << "set ylabel \"Packets [1]\"" << endl;
               *gpScript << "set xlabel \"Time [s]\"" << endl;
               *gpScript << "plot \"" << dataName << "\" using 1:"
                         << 1 + 2 * (layers + 1) << " title "
                         << "\"Total\" with lines";
               for(cardinal i = 0;i < layers;i++) {
                  *gpScript << ", \"" << dataName << "\" using 1:"
                            << 1 + (layers + 1) + (i + 1) << " title "
                            << "\"Layer #" << i << "\" with lines";
               }
               *gpScript << endl;

               *gpScript << "set title \"RTP Trace Client " << address << " - Packets Lost\" \"Times-Roman,16\"" << endl;
               *gpScript << "set timestamp \"" << infoString << "\" 0,0 \"Times-Roman,12\"" << endl;
               *gpScript << "set ylabel \"Packets Lost [1]\"" << endl;
               *gpScript << "set xlabel \"Time [s]\"" << endl;
               *gpScript << "plot \"" << dataName << "\" using 1:"
                         << 1 + 3 * (layers + 1) << " title "
                         << "\"Total\" with lines";
               for(cardinal i = 0;i < layers;i++) {
                  *gpScript << ", \"" << dataName << "\" using 1:"
                            << 1 + 2 * (layers + 1) + (i + 1) << " title "
                            << "\"Layer #" << i << "\" with lines";
               }
               *gpScript << endl;

               *gpScript << "set title \"RTP Trace Client " << address << " - Fraction Lost\" \"Times-Roman,16\"" << endl;
               *gpScript << "set timestamp \"" << infoString << "\" 0,0 \"Times-Roman,12\"" << endl;
               *gpScript << "set ylabel \"Fraction Lost [%]\"" << endl;
               *gpScript << "set xlabel \"Time [s]\"" << endl;
               *gpScript << "plot \"" << dataName << "\" using 1:"
                         << 1 + 4 * (layers + 1) << " title "
                         << "\"Total\" with lines";
               for(cardinal i = 0;i < layers;i++) {
                  *gpScript << ", \"" << dataName << "\" using 1:"
                            << 1 + 3 * (layers + 1) + (i + 1) << " title "
                            << "\"Layer #" << i << "\" with lines";
               }
               *gpScript << endl;

               *gpScript << "set title \"RTP Trace Client " << address << " - Interarrival Jitter\" \"Times-Roman,16\"" << endl;
               *gpScript << "set timestamp \"" << infoString << "\" 0,0 \"Times-Roman,12\"" << endl;
               *gpScript << "set ylabel \"Interarrival Jitter [us]\"" << endl;
               *gpScript << "set xlabel \"Time [s]\"" << endl;
               *gpScript << "plot \"" << dataName << "\" using 1:"
                         << 1 + 5 * (layers + 1) << " title "
                         << "\"Average\" with lines";
               for(cardinal i = 0;i < layers;i++) {
                  *gpScript << ", \"" << dataName << "\" using 1:"
                            << 1 + 4 * (layers + 1) + (i + 1) << " title "
                            << "\"Layer #" << i << "\" with lines";
               }
               *gpScript << endl;
               return;
            }
            else {
               cerr << "ERROR: Unable to create data file " << dataName << "!" << endl;
            }
            delete gpData;
         }
      }
      else {
         cerr << "ERROR: Unable to create script file " << scriptName << "!" << endl;
      }
      delete gpScript;
   }
   exit(1);
}


// ###### Main program ######################################################
int main(int argc, char* argv[])
{
   // ===== Check arguments =================================================
   if(argc < 2) {
      cerr << "Usage: " << argv[0] << endl
           << "[URL] {-local=host{:Port}} {-debug} {-null} {-encoding=number} {-priority=priority} {sessionID=id} {sessionpriority=priority} {-maxdelay=delay} {-bwmin=bandwidth} {-bwmax=bandwidth} {-prefix=name} {-layers=layers} {-info=infostring} {-force-ipv4}" << endl;
      exit(0);
   }
   bool     optForceIPv4    = false;
   cardinal encoding        = 0;
   char*    receiverName    = NULL;
   char*    info            = "";
   char*    prefix          = NULL;
   int8     streamPriority  = 0;
   int8     sessionPriority = 0;
   card32   sessionID       = 0;
   cardinal bwMin           = 0;
   cardinal bwMax           = (cardinal)-1;
   cardinal layers          = 3;
   double   maxDelay        = HUGE_VAL;
   for(cardinal i = 1;i < (cardinal)argc;i++) {
      if(!(strcasecmp(argv[i],"-force-ipv4"))) optForceIPv4 = true;
      else if(!(strncasecmp(argv[i],"-prefix=",8))) prefix       = &argv[i][8];
      else if(!(strncasecmp(argv[i],"-info=",6)))   info         = &argv[i][6];
      else if(!(strncasecmp(argv[i],"-encoding=",10)))  encoding = atol(&argv[i][10]);
      else if(!(strncasecmp(argv[i],"-priority=",10)))         streamPriority   = (int8)atol(&argv[i][10]);
      else if(!(strncasecmp(argv[i],"-sessionID=",11)))        sessionID        = (card32)atol(&argv[i][11]);
      else if(!(strncasecmp(argv[i],"-sessionpriority=",16)))  sessionPriority  = (int8)atol(&argv[i][16]);
      else if(!(strncasecmp(argv[i],"-bwmin=",7)))      bwMin    = atol(&argv[i][7]);
      else if(!(strncasecmp(argv[i],"-bwmax=",7)))      bwMax    = atol(&argv[i][7]);
      else if(!(strncasecmp(argv[i],"-maxdelay=",10)))  maxDelay = atof(&argv[i][10]);
      else if(!(strncasecmp(argv[i],"-layers=",8)))     layers   = atol(&argv[i][8]);
      else if(argv[i][0] == '-') {
         cerr << "Wrong parameter: " << argv[i] << endl;
         exit(1);
      }
   }
   if(optForceIPv4) {
      if(InternetAddress::UseIPv6 == true) {
         InternetAddress::UseIPv6 = false;
         cerr << "NOTE: IPv6 support disabled!" << endl;
      }
   }
   if(bwMin > bwMax) {
      bwMax = bwMin;
   }


   // ====== Get server address and media name ==============================
   String protocol;
   String host;
   String path;
   bool ok = scanURL(argv[1],protocol,host,path);
   if((ok == false) || (protocol != "rtpt")) {
      cerr << "ERROR: Invalid URL! Check URL and try again." << endl;
      cerr << "       Example: rtpt://gaffel:7500/Test.tdtf" << endl;
      cleanUp(1);
   }
   InternetAddress serverAddress(host);
   if(!serverAddress.isValid()) {
      cerr << "ERROR: Invalid server address! Check URL and try again." << endl;
      cleanUp(1);
   }


   // ====== Initialize Trace client ========================================
   client = new TraceClient(receiverName);
   if(client == NULL) {
      cerr << "ERROR: Client::main() - Out of memory!" << endl;
      cleanUp(1);
   }
   client->setEncoding(encoding);
   client->setStreamPriority((int8)streamPriority);
   client->setSessionPriority((int8)sessionPriority);
   client->setMinWantedBandwidth(bwMin);
   client->setMaxWantedBandwidth(bwMax);
   client->setMaxWantedDelay(maxDelay);


   // ====== Start playing ==================================================
   ok = client->play(host.getData(),path.getData(),sessionID);
   if(ok == false) {
      cerr << "ERROR: Client::main() - TraceClient::play() failed!" << endl;
      cleanUp(1);
   }
#ifndef FAST_BREAK
   installBreakDetector();
#endif

   if(prefix != NULL) {
      initGNUplot(prefix,info,layers,client->getOurAddressString(InternetAddress::PF_Full));
   }


   // ====== Print status ===================================================
   cout << "RTP Trace Client - Copyright (C) 2000 Thomas Dreibholz" << endl;
   cout << "------------------------------------------------------" << endl;
   cout << endl;
   cout << "Server address:    " << client->getServerAddressString() << endl;
   cout << "Client address:    " << client->getOurAddressString() << endl;
   cout << endl;
   char str[32];
   snprintf((char*)&str,sizeof(str),"$%08x",client->getOurSSRC());
   cout << "Client SSRC:       " << str                      << endl;
   cout << "Stream Priority:   " << (integer)streamPriority  << endl;
   cout << "Session ID:        " << (cardinal)sessionID      << endl;
   cout << "Stream Priority:   " << (integer)sessionPriority << endl;
   cout << "Maximum Delay:     " << (maxDelay / 1000.0) << " [ms]" << endl;
   cout << "Minimum bandwidth: " << bwMin  << endl;
   cout << "Maximum bandwidth: " << bwMax  << endl;
   cout << "Media Name:        " << path   << endl;
   cout << "Layers:            " << layers << endl;
   cout << endl;


   // ====== Main loop ======================================================
   cardinal EOFRepeatDelay = 0;
   card64   oldBytes[layers];
   card64   oldPackets[layers];
   card64   oldPacketsLost[layers];
   for(cardinal i = 0;i < layers;i++) {
      oldBytes[i] = oldPackets[i] = oldPacketsLost[i] = 0;
   }
   card64   startTS    = getMicroTime() - 1;
   card64   oldTS      = startTS;         // avoid now=oldTS (-> divide by 0)
   while(client->playing()) {
      // ====== Check for errors ============================================
      const card8 error = client->getErrorCode();
      switch(error) {
         case ME_NoError:
         case ME_NoMedia:
          break;
         case ME_EOF:
            EOFRepeatDelay++;
            if(EOFRepeatDelay > 1) {
               EOFRepeatDelay = 0;
               client->setPosition(0);
            }
          break;
         default:
            cerr << endl << "Encoder error while playing: #" << (int)error << "." << endl;
            goto cleanUp;
          break;
      }
#ifndef FAST_BREAK
      if(breakDetected()) {
         goto cleanUp;
      }
#endif


      // ====== Print information ===========================================
      const card64 position = client->getPosition();
      const card64 seconds  = position / PositionStepsPerSecond;
      card64 bytes[layers];
      card64 packets[layers];
      card64 packetsLost[layers];
      double fractionLost[layers];
      double jitter[layers];
      for(cardinal i = 0;i < layers;i++) {
         bytes[i]        = client->getBytesReceived(i);
         packets[i]      = client->getPacketsReceived(i);
         packetsLost[i]  = client->getPacketsLost(i);
         fractionLost[i] = client->getFractionLost(i);
         jitter[i]       = client->getJitter(i);
      }
      const cardinal        currentLayers  = client->getLayers();
      const card64          now            = getMicroTime();
      const Range<cardinal> bandwidthRange = client->getBandwidth();
      printf("\x0d%2Ld:%02Ld.%02Ld   [%7d <= %7d <= %7d BPS] [Quality: %1.1f FPS, %1.2f Utilization, Priority %d]  [%s]      ",
             (seconds / 60),(seconds % 60),
             (position % PositionStepsPerSecond) / (PositionStepsPerSecond / 100),
             bandwidthRange.getMin(),
             bandwidthRange.getValue(),
             bandwidthRange.getMax(),
             client->getFrameRate(),
             client->getUtilization() * 100.0,
             client->getStreamPriority(),
             client->getEncoding());
      fflush(stdout);


      // ====== Write GNUplot output ========================================
      if((gpData != NULL) && (currentLayers > 0)) {
         card64 oldBytesTotal = 0;
         card64 bytesTotal    = 0;
         *gpData << (double)(now - startTS) / 1000000.0 << " ";
         for(cardinal i = 0;i < layers;i++) {
            oldBytesTotal += oldBytes[i];
            bytesTotal    += bytes[i];
            const card64 bps = (bytes[i] - oldBytes[i]) * 1000000 / (now - oldTS);
            *gpData << bps << " ";
         }
         *gpData << (bytesTotal - oldBytesTotal) * 1000000 / (now - oldTS) << " ";

         card64 oldPacketsTotal = 0;
         card64 packetsTotal    = 0;
         for(cardinal i = 0;i < layers;i++) {
            oldPacketsTotal += oldPackets[i];
            packetsTotal    += packets[i];
            const card64 pps = (packets[i] - oldPackets[i]) * 1000000 / (now - oldTS);
            *gpData << pps << " ";
         }
         *gpData << (packetsTotal - oldPacketsTotal) * 1000000 / (now - oldTS) << " ";

         card64 oldPacketsLostTotal = 0;
         card64 totalPacketsLost    = 0;
         for(cardinal i = 0;i < layers;i++) {
            oldPacketsLostTotal += oldPacketsLost[i];
            totalPacketsLost += packetsLost[i];
            *gpData << (packetsLost[i] - oldPacketsLost[i]) << " ";
         }
         *gpData << (totalPacketsLost - oldPacketsLostTotal) << " ";

         double avgFractionLost = 0.0;
         for(cardinal i = 0;i < layers;i++) {
            avgFractionLost += fractionLost[i];
            *gpData << 100.0 * fractionLost[i] << " ";
         }
         avgFractionLost /= (double)currentLayers;
         *gpData << avgFractionLost * 100.0 << " ";

         double avgJitter = 0.0;
         for(cardinal i = 0;i < layers;i++) {
            avgJitter += jitter[i];
            *gpData << jitter[i] << " ";
         }
         avgJitter /= (double)currentLayers;
         *gpData << avgJitter;

         *gpData << endl;
      }


      // ====== Wait ========================================================
      for(cardinal i = 0;i < layers;i++) {
         oldBytes[i]       = bytes[i];
         oldPackets[i]     = packets[i];
         oldPacketsLost[i] = packetsLost[i];
      }
      oldTS = now;
      Thread::delay(1000000,true);
   }


   // ====== Clean up =======================================================
cleanUp:
   cleanUp(0);
}
