// ##########################################################################
// ####                                                                  ####
// ####                    Master Thesis Implementation                  ####
// ####  Management of Layered Variable Bitrate Multimedia Streams over  ####
// ####                  DiffServ with A Priori Knowledge                ####
// ####                                                                  ####
// #### ================================================================ ####
// ####                                                                  ####
// ####                                                                  ####
// #### Utility Function Implementation                                  ####
// ####                                                                  ####
// #### Version 1.00  --  October 10, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 2000 Thomas Dreibholz                              ####
// #### University of Bonn, Department of Computer Science IV            ####
// #### EMail: Dreibholz@bigfoot.com                                     ####
// #### WWW:   http://www.bigfoot.com/~dreibholz/diplom/index.html       ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "utilityfunction.h"


namespace Coral {


// ###### Evaluate utility function using translated constants ##############
double evaluateUtilityFunction(const cardinal type,
                               const double   scaleFactor,
                               const double*  constantArray,
                               const cardinal constants)
{
   if(scaleFactor <= 0.0) {
      return(0.0);
   }
   else if(scaleFactor >= 1.0) {
      return(1.0);
   }

   double utilization = 0.0;
   switch(type) {

      // ====== Linear utilization ==========================================
      case UF_Linear:
         return(scaleFactor);
       break;

      // ====== Exponential utilization #1 ==================================
      case UF_Exponential1:
         {
            if(constants < 2) {
               cerr << "WARNING: evaluateUtilityFunction() - Too few constants!"
                    << endl;
            }

            else {
               // ====== Do exponential calculation, if possible ============
               const double b = (constantArray[1] * log(-0.50 + 1.00) -
                                 constantArray[0] * log(-0.95 + 1.00)) /
                                    (constantArray[1] - constantArray[0]);
               const double a = (log(-0.50 + 1.00) - b) / constantArray[0];
               utilization = 1.0 - exp(a * scaleFactor + b);
               if(utilization < 0.0) {
                  utilization = 0.0;
               }
            }
         }
       break;

      // ====== Exponential utilization #2 ==================================
      case UF_Exponential2:
         {
            if(constants < 1) {
               cerr << "WARNING: evaluateUtilityFunction() - Too few constants!"
                    << endl;
            }

            else {
               // ====== Do exponential calculation, if possible ============
               const double a = 1.0 / (constantArray[0] - 10.0);
               const double b = exp(1 / a) - 1.0;
               const double c = 2.0 - exp(1 / a);
               utilization = a * log(b * (scaleFactor + 1.0) + c);
            }
         }
       break;

      // ====== Unknown utilization =========================================
      default:
         cerr << "WARNING: evaluateUtilityFunction() - Unimplemented utility function type "
              << type << "!" << endl;
       break;
   }

   if(utilization > 1.0) {
      cerr << "WARNING: evaluateUtilityFunction() - Calculated utilization "
           << utilization << " too high!" << endl;
      cerr << "ScaleFactor = " << scaleFactor << endl;
      for(cardinal i = 0;i < constants;i++) {
         cerr << "C" << i << " = " << constantArray[i] << endl;
      }
      utilization = 1.0;
   }
   else if(utilization < 0.0) {
      cerr << "WARNING: evaluateUtilityFunction() - Calculated utilization "
           << utilization << " too low!" << endl;
      cerr << "ScaleFactor = " << scaleFactor << endl;
      for(cardinal i = 0;i < constants;i++) {
         cerr << "C" << i << " = " << constantArray[i] << endl;
      }
      utilization = 0.0;
   }
   return(utilization);
}


}
